//tema_main.c

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <sys/time.h>
#include "mrtu.h"
#include "main.h" 		//подключаем наш заголовочный файл 
#include "rsocket.h"		//подключаем заголовочный файл для работы с сокетами

int	ipport;	//наш порт прослушивания

int	num_work;	//сётчик для работы
/*
************************
массив рабочих регистров
************************
*/
I16	regs[_cnt_regs];
I16	motors[4];
I16	servos[18];

int main(int argc, char **argv)
{
	int scnt = 1; //счётчик для анализа входных параметров
	int res = 1;
	int cnt;
	char	serialname[255] = {0};
	//без параметров
	if(argc < 2)
	{
		printf("\nParametrs not defined!\r\n");
		printf("\"--help\" for information\n");
		return 1;
	}
	//
	while(scnt < argc)
	{
		//подсказка
		if(strcmp(argv[scnt], "--help") == 0)
		{
			printf(__help);
			return 0;
		}
		//порт TCP
                else if(strcmp(argv[scnt], "-p") == 0)
                {
                        scnt++;
                        if(!(scnt < argc))
                                return 2;
                        ipport = atoi(argv[scnt]);
                        if(ipport == 0)
				return 3;
		}
		//последовательный порт
		else if(strcmp(argv[scnt], "-sp") == 0)
                {
                        scnt++;
                        if(!(scnt < argc))
                                return 2;
                        strcpy(&serialname[0], argv[scnt]);
		}
		else
			return 4;
		scnt++;
	}
	//открытие прослушивающего сокета - запуск сервера
	res = SocketStart();
	printf("---- opening tcp/ip  %u ----\r\n", ipport);
	if(res != 0)
	{
		printf("ERROR starting server\n");
		return 5;
	}
	//соединяемся с Arduino
	res = MRTU_Connect(&serialname[0]);
	if(res != 0)
	{
		SocketStop();
		printf("ERROR connecting to Arduino\n");
		return 5;
	}
	//обнуляем рабочий счётчик
	num_work = 0;
	//моторы
	cnt = 0;
	while(cnt < 4)
	{
		motors[cnt] = 0;
		cnt++;
	}
	//сервомашины
	cnt = 0;
	while(cnt < 18)
	{
		servos[cnt] = 0;
		cnt++;
	}
	//наш цикл Main
	while(1)
	{
		MainWork();	//Рабочая функция
		usleep(50000);	//ожтдание (uS)
	}
	//
	return 0;
}

//рабочий модуль
void	MainWork(void)
{
	int	count;
	int	addr;
	int	cnt;
	//
	pthread_mutex_lock(&ServerMutex);
	//копирование заданий регистров
	addr 	= 0;
	count	= 0;
	while(count < _cnt_regs)
	{
		regs[count] = (*(I16*)(&wbuffrom[addr]));
		addr += 2;	
		count++;
	}
	//копировние значений регистров
	addr 	= 0;
	count	= 0;
	while(count < _cnt_regs)
	{
		(*(I16*)(&wbufto[addr])) = regs[count];
		addr += 2;	
		count++;
	}
	switch(regs[_mode_contrl])
	{
		//приращение на единицу
		case _mode_ctrl1:
			//моторы
			cnt = 0;
			while(cnt < 4)
			{
				if((motors[cnt] < regs[_num_vol_m1 + cnt]) &&
				(motors[cnt] < 255))
					motors[cnt]++;
				else if((motors[cnt] > regs[_num_vol_m1 + cnt]) &&
				(motors[cnt] > -255))
					motors[cnt]--;
				cnt++;
			}
			//сервомашины
			cnt = 0;
			while(cnt < 18)
			{
				if((servos[cnt] < regs[_num_pos_s1 + cnt]) &&
				(servos[cnt] < 180))
					servos[cnt]++;
				else if((servos[cnt] > regs[_num_pos_s1 + cnt]) &&
				(servos[cnt] > -180))
					servos[cnt]--;
				cnt++;
			}
			break;
		//обычная установка
		default:
			//копирование данных для реализации
			memcpy(&motors[0], &regs[_num_vol_m1], 8);	//моторы
			memcpy(&servos[0], &regs[_num_pos_s1], 36);	//сервомашины
			break;
	}
	pthread_mutex_unlock(&ServerMutex);
	//
	switch(num_work)
	{
		case 0:
			MRTU_CMD_16(1, _num_vol_m1, (U16*)(&motors[0]), 4);
			num_work++;
			break;
		case 1:	//
			//MRTU_CMD_6(1, _num_pos_s1, (U16*)(&servos[0]), 2);
			MRTU_CMD_6(1, _num_pos_s1, servos[0]);
			num_work ++;
			break;
		case 2:	//
			MRTU_CMD_6(1, _num_pos_s2, servos[1]);
			num_work = 0;
			break;
		default:
			num_work = 0;
			break;
	}
}
